library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;


entity Labx1 is
generic
(
	TAM_DATA	: integer := 8; 	-- Numero de bits do dado a ser enviado para o LCD
	NUM_DATA	: integer := 32;	-- Numero de dados que serão enviados
	-- Matriz LEDs
	--PRESCALER : integer := 25000000;
	PRESCALER : integer := 4000;
	--UART
   CLOCK_DIVIDER		: integer := 50;
	CLOCK_FREQUENCY	: integer := 50000000; 	-- Frequencia do clock em Hz
	BAUDRATE_RX 		: integer := 115200;		-- Frequencia da recepção UART em bps
	BAUDRATE_TX 		: integer := 115200;		-- Frequencia do envio UART em bps
	--LCD
	--Valores reais
	DIV_NUM		: integer := 500;		-- 50Mhz/500 = 100KHz = 10us
	BOOT_TIME	: integer := 2000;  	-- 2000/100KHz = 20ms
	WR_TIME		: integer := 5;		-- alterado, era 5	-- 5*10us = 50us
	CLR_TIME		: integer := 200;		-- 2ms 
   
   LED_RGB_DELAY  : integer := 12500000
);
port
(
   CLOCK_50MHz 	: in	std_logic;
	SW			   	: in 	std_logic_vector(3 downto 0);    --operandos(15 downto 8 => operando2, 7 downto 0 => operando1)
	KEY		   	: in 	std_logic_vector(11  downto 0);  --comando(3=>'start_send',0=>'rst');
   -- Interface UART
	UART_TXD			: out std_logic;
	-- LED RGB
   LED_R          : out std_logic;
   LED_G          : out std_logic;
   LED_B          : out std_logic;
   -- LCD
	LCD_D		      : inout	std_logic_vector(7  downto 0); 	-- LCD data is a bidirectional bus...
	LCD_RS 	   	: out   std_logic;                		-- LCD register select
	LCD_RW  	   	: out   std_logic;                		-- LCD Read / nWrite
	LCD_EN  		   : out   std_logic;                		-- LCD Enable
	LCD_BACKLIGHT	: out   std_logic;
   -- Matriz de LEDs
   LEDM_C		:		out std_logic_vector(4 downto 0);
	LEDM_R		:		out std_logic_vector(7 downto 0)
);
end Labx1;

architecture behavior of Labx1 is

signal LED_RGB 		 : std_logic_vector (2 downto 0);
signal w_reset_50mhz  : std_logic;
signal w_reset_2khz   : std_logic;
signal start_tx		 : std_logic;
signal busy_tx 		 : std_logic;
signal serial_write	 : std_logic_vector(7 downto 0);
signal idx				 : integer range 1 to 32;
signal char 			 : std_logic_vector(4 downto 0);
signal w_50mhz_clk	 : std_logic;
signal w_2khz_clk		 : std_logic;


-- Declaração da interface serial
COMPONENT uart
	GENERIC (   CLOCK_FREQUENCY   : INTEGER;
               BAUDRATE_RX       : INTEGER;
               BAUDRATE_TX       : INTEGER;
               CLOCK_DIVIDER     : INTEGER);
	PORT
	(
		CLK_UART_i		:	 IN STD_LOGIC;
		serial_rx_i		:	 IN STD_LOGIC;
		start_tx_i		:	 IN STD_LOGIC;
		serial_write_i	:	 IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		busy_rx_o		:	 OUT STD_LOGIC;
		busy_tx_o		:	 OUT STD_LOGIC;
		valid_rx_o		:	 OUT STD_LOGIC;
		serial_tx_o		:	 OUT STD_LOGIC;
		serial_read_o	:	 OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
	);
END COMPONENT;

-- Declaração PLL
COMPONENT my_altpll
   PORT (
		inclk0	 : IN  STD_LOGIC; -- OSC 50MHZ
		c0	       : OUT STD_LOGIC; -- 50MHZ
		c2	       : OUT STD_LOGIC  -- 2KHZ
	);
END COMPONENT;

-- Declaração sincronizador de reset
COMPONENT reset_sync
   PORT
	(
		i_clk	               :	 IN STD_LOGIC;
		i_external_reset_n	:	 IN STD_LOGIC;
		o_reset_n      		:	 OUT STD_LOGIC;
		o_reset		         :	 OUT STD_LOGIC
   );
END COMPONENT;

	-- Declaraçao da memória que armazenara os caracteres a serem Enviados via UART
	type MEM is array (1 to NUM_DATA) of std_logic_vector(TAM_DATA downto 1);
	signal word: MEM := MEM'(
			x"20", -- 	
			x"20", -- 	
			x"4D", -- M	
			x"41", -- A	
			x"43", -- C 
			x"4E", -- N	
			x"49", -- I	
			x"43", -- C	
			x"41", -- A	
			x"20", -- 				
			x"44", -- D
			x"48", -- H	
			x"57", -- W	
         x"20", -- 	
			x"20", -- 	
			x"20", -- 	   ultimo caractere da linha 1
			x"20", -- 	
			x"20", -- 	
			x"20", -- 
			x"20", -- 	
			x"20", -- 	
			x"41", -- A	
			x"4C", -- L	
			x"54", -- T	
			x"45", -- E	
			x"52", -- R	
			x"41", -- A	
			x"20", -- 	
			x"20", -- 	
			x"20", -- 	
			x"20", -- 
			x"20"  -- 
	);

begin





-- controle do LED_RGB   
process (w_50mhz_clk, w_reset_50mhz)
   
   variable count : integer range 1 to LED_RGB_DELAY := 1;
   
   begin
   if  w_reset_50mhz = '1' then
      LED_RGB  <= "000";
      count    := 1;
   elsif rising_edge(w_50mhz_clk) then
      count := count + 1;
      if count = LED_RGB_DELAY then
         count := 1;
         LED_RGB <= LED_RGB + "001";
      end if;
      if LED_RGB = "111" then
         LED_RGB <= "001";
      end if;
   end if;
end process;

LED_R <= LED_RGB(0);
LED_G <= LED_RGB(1);
LED_B <= LED_RGB(2);

-- Instanciaçao dos componentes

   my_altpll_inst : my_altpll 
   PORT MAP (
		inclk0	=> CLOCK_50MHz,
		c0	      => w_50mhz_clk,
      c2	      => w_2khz_clk
	);


	reset_synch_50mhz_inst : reset_sync
	PORT MAP (
		i_clk                => w_50mhz_clk,
		i_external_reset_n   => KEY(0),
		o_reset		         => open,
		o_reset_n            => w_reset_50mhz
	);

   reset_synch_2khz_inst : reset_sync
	PORT MAP (
		i_clk                => w_2khz_clk,
		i_external_reset_n   => KEY(0),
		o_reset		         => open,
		o_reset_n            => w_reset_2khz
	);

   -- Instanciaçao LCD
	inst_lcd_top : entity work.display_top_component_Mercurio
	generic map(
		TAM_DATA 	=> TAM_DATA,
		NUM_DATA	   => NUM_DATA,
		--LCD
		DIV_NUM		=> DIV_NUM,
		BOOT_TIME	=> BOOT_TIME,
		WR_TIME		=> WR_TIME,
		CLR_TIME	   => CLR_TIME
	)
	port map (
		CLOCK_50MHz	=> w_50mhz_clk,
      RESET_50MHz => w_reset_50mhz,
		SW		      => SW,
      KEY         => KEY,
		-- LCD
		LCD_D 	     => LCD_D,
		LCD_RS 	     => LCD_RS,
		LCD_RW  	     => LCD_RW,
		LCD_EN        => LCD_EN,
		LCD_BACKLIGHT => LCD_BACKLIGHT
	);	
	
   -- Instanciaçao Matriz de LEDs
	inst_matriz_leds  : entity work.matriz_leds
   generic map ( PRESCALER => 1 )
	port map (
		clk_i		=> w_2khz_clk,
		rst_i		=> w_reset_2khz,
		idx_i		=> char,
		col_o		=> LEDM_C,
		row_o		=> LEDM_R
	);
	
	-- Processo para fazer com que a matriz de LEDs apresente a frase:
	-- DHW-ALTERA 
	process(w_2khz_clk, w_reset_2khz)
	variable cnt: integer range 0 to PRESCALER;
	begin
		if (w_reset_2khz = '1') then
			cnt := 0;
			char <= (others => '0');
		elsif(rising_edge(w_2khz_clk)) then
			if	cnt = PRESCALER then
				cnt := 0;
				char <= char + "0001";
				if char = "10001" then
					char <= "00000";
				end if;
			else
				cnt := cnt + 1;
			end if;			
		end if;
	end process;
	
	-- Instanciaçao da interface serial
	uart_inst: uart
	GENERIC MAP (  CLOCK_FREQUENCY   => CLOCK_FREQUENCY,
                  BAUDRATE_RX       => BAUDRATE_RX,
                  BAUDRATE_TX       => BAUDRATE_TX,
                  CLOCK_DIVIDER     => CLOCK_DIVIDER)
	PORT MAP
	(
		CLK_UART_i		=> w_50mhz_clk,
		serial_rx_i		=> '1',
		start_tx_i		=> start_tx,
		serial_write_i	=> serial_write,
		busy_rx_o		=> open,
		busy_tx_o		=> busy_tx,
		valid_rx_o		=> open,
		serial_tx_o		=> UART_TXD,
		serial_read_o	=> open
	);
	
   	-- Divisão do clock para 1MHz - sinal de enable
	-- Clock_div_1MHz:
	-- process (CLOCK_50MHz)
	-- begin
	-- if rising_edge(CLOCK_50MHz) then
		-- if div_clock = (CLOCK_DIVIDER/2) then
			-- div_clock <= 1;
			-- ENABLE_1MHz <= '1';
		-- else
			-- div_clock <= div_clock + 1;
         -- ENABLE_1MHz <= '0';
		-- end if;
	-- end if;
	-- end process;
   
	-- Processo para o envio da frase:
	-- DHW - Altera Kit Mercurio IV, que está gravada na memória
	Data_send:
	process (w_50mhz_clk, w_reset_50mhz)
	variable count : integer range 0 to CLOCK_FREQUENCY - 1:= 0;
	begin
		if w_reset_50mhz = '1' then
			count := 0;
			idx <= 1;
			start_tx <= '0';
			serial_write <= (others => '0');
		elsif w_50mhz_clk'event and w_50mhz_clk = '1' then
			if busy_tx = '0' and count = CLOCK_FREQUENCY/32 then
				count := 0;
				serial_write <= word(idx);
				start_tx <= '1';
				idx <= idx + 1;
				if idx = 32 then
					idx <= 1;	
				end if;
			elsif start_tx = '1' then
				start_tx <= '0';
				count := count + 1;
			else
				count := count + 1;
			end if;
		end if;
	end process;
	
end behavior;